from configparser import ConfigParser
from pathlib import Path
from typing import Any

from .file import ConfigFile


class INIFile(ConfigFile):
    def __init__(self, path: str | Path) -> None:
        if isinstance(path, str):
            path = Path(path)
        self._path = path
        self._config_parser = ConfigParser()

    def exists(self) -> bool:
        return self._path.exists()

    def read(self) -> dict[str, Any]:
        self._config_parser.read(self._path)
        return dict(self._config_parser.items("ENDTIME"))

    def write(self, config: dict[str, Any]) -> None:
        if self.exists() is False:
            self._path.parent.mkdir(parents=True, exist_ok=True)
            self._path.touch(mode=0o600)

        self._config_parser["ENDTIME"] = config

        with open(self._path, "w", encoding="utf-8", newline="") as config_file:
            self._config_parser.write(config_file)



from typing import Any


class ConfigFile:
    def exists(self) -> bool:
        raise NotImplementedError()

    def read(self) -> dict[str, Any]:
        raise NotImplementedError()

    def write(self, config: dict[str, Any]) -> None:
        raise NotImplementedError()



from __future__ import annotations

from copy import deepcopy
from pathlib import Path
from typing import Any

from platformdirs import user_config_path

from .file import ConfigFile
from .ini_file import INIFile


CONFIG_DIR = Path(user_config_path("lifetimer", appauthor=False, roaming=True))

_default_config: Config | None = None


class Config:
    default_config: dict[str, Any] = {
        "year": 0,
        "month": 12,
        "day": 31,
        "hour": 23,
        "minute": 59,
        "second": 59,
    }

    def __init__(self) -> None:
        self._config = deepcopy(self.default_config)

    @classmethod
    def create(cls) -> Config:
        global _default_config

        if _default_config is None:
            _default_config = cls()
            config_file = INIFile(CONFIG_DIR / "config.ini")
            _default_config.set_config_source_file(config_file)

            if config_file.exists():
                _default_config.merge(config_file.read())

        return _default_config

    def get(self, setting_name: str, default: Any = None) -> Any:
        return self._config[setting_name] or default

    def set(self, setting_name: str, value: str | int) -> None:
        self._config[setting_name] = value

    def merge(self, config: dict[str, Any]) -> None:
        for key in config.keys():
            self._config[key] = config[key]

    def set_config_source_file(self, file: ConfigFile) -> None:
        self._file = file

    def save(self) -> None:
        self._file.write(self._config)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['lifetimer', 'lifetimer.config']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0', 'platformdirs>=3.0.0,<4.0.0', 'yaspin>=2.3.0,<3.0.0']

entry_points = \
{'console_scripts': ['lifetimer = lifetimer.cli:execute_from_command_line']}

setup_kwargs = {
    'name': 'lifetimer',
    'version': '0.1.0',
    'description': 'Lifespan timer',
    'long_description': "# Lifetimer\n\n[![PyPI version](https://img.shields.io/pypi/v/lifetimer)](https://pypi.org/project/lifetimer/)\n[![Python Versions](https://img.shields.io/pypi/pyversions/lifetimer)](https://pypi.org/project/lifetimer/)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/licenses/MIT)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\nThis is a lifespan timer that allows you to watch the seconds tick down to the time you set for yourself.\n\nIt's for anyone who wants to feel the time dwindle, push themselves to get up to speed on their studies and work.\n\n## Installation\n\n```sh\npip install lifetimer\n```\n\n## Usage\n\nWhen installed, you have to set your last date and time of your life through `lifetimer` or `lifetimer init` commands.\n\nIf you want to change the settings, use `lifetimer init` command.\n\n```sh\n$ lifetimer init\nPlease, enter your last date and time of your life.\nYear: 2100\nMonth [12]: 3\nDay [31]: 6\nHour [23]: 17\nMinute [59]: 58\nSecond [59]: 50\n```\n\nAfter settings, when you want to check your lifespan, just type `lifetimer` command.\n\n```sh\n$ lifetimer\n🕛 You have 2,455,857,087 seconds to live.\n```\n",
    'author': 'kimfame',
    'author_email': 'renownkim@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/kimfame/lifetimer.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.11,<4.0',
}


setup(**setup_kwargs)



from datetime import datetime


class Lifetimer:
    def __init__(
        self,
        year: int,
        month: int = 12,
        day: int = 31,
        hour: int = 23,
        minute: int = 59,
        second: int = 59,
    ) -> None:
        self.end_day = self.validate_datetime(
            year,
            month,
            day,
            hour,
            minute,
            second,
        )

    def __str__(self) -> str:
        if self.get_lifespan() == 0:
            return "You get another life!"
        else:
            return f"You have {self.get_lifespan():,} seconds to live."

    def get_lifespan(self) -> int:
        today = datetime.today()

        if today < self.end_day:
            delta = self.end_day - today
            total_seconds = round(delta.total_seconds())
            return total_seconds
        else:
            return 0

    @classmethod
    def validate_datetime(
        self,
        year: int,
        month: int,
        day: int,
        hour: int,
        minute: int,
        second: int,
    ) -> datetime:
        try:
            dt = datetime(
                year=year,
                month=month,
                day=day,
                hour=hour,
                minute=minute,
                second=second,
            )
            delta = dt - datetime.today()
            total_seconds = round(delta.total_seconds())

            if total_seconds > 9223372036:
                raise OverflowError
            if total_seconds < 0:
                raise ValueError

            return dt
        except ValueError:
            raise ValueError("Invalid datetime value")
        except TypeError:
            raise TypeError("Invalid datetime type")
        except OverflowError:
            raise OverflowError("Too large to count the datetime")



import signal
import sys
import time

import click

from datetime import datetime
from pathlib import Path
from typing import Optional

from yaspin import yaspin, Spinner

from .lifetimer import Lifetimer
from .config.config import Config


def init() -> int:
    print("Please, enter your last date/time of your life.")
    while True:
        year = click.prompt("Year", type=click.IntRange(datetime.today().year, 9999))
        month = click.prompt("Month", default=12, type=click.IntRange(1, 12))
        day = click.prompt("Day", default=31, type=click.IntRange(1, 31))
        hour = click.prompt("Hour", default=23, type=click.IntRange(0, 23))
        minute = click.prompt("Minute", default=59, type=click.IntRange(0, 59))
        second = click.prompt("Second", default=59, type=click.IntRange(0, 59))

        try:
            Lifetimer.validate_datetime(
                year=year,
                month=month,
                day=day,
                hour=hour,
                minute=minute,
                second=second,
            )
        except ValueError:
            print("Please, check again your last date/time.")
            continue
        except OverflowError:
            print("It's too large to count your lifespan.")
            continue
        else:
            break

    config = Config.create()
    config.set("year", year)
    config.set("month", month)
    config.set("day", day)
    config.set("hour", hour)
    config.set("minute", minute)
    config.set("second", second)
    config.save()

    return 0


class Command:
    def __init__(self, argv: Optional[str] = None) -> None:
        self.argv = argv or sys.argv[:]
        self.prog_name = Path(self.argv[0]).name
        if self.prog_name == "__main__.py":
            self.prog_name = "python -m lifetimer"

    def execute(self) -> int:
        if len(self.argv) > 1 and self.argv[1] == "init":
            return init()

        config = Config.create()

        if config.get("year") is None:
            init()

        lifetimer = Lifetimer(
            int(config.get("year")),
            int(config.get("month", 12)),
            int(config.get("day", 31)),
            int(config.get("hour", 23)),
            int(config.get("minute", 59)),
            int(config.get("second", 59)),
        )

        sp = Spinner(
            [
                "🕛",
                "🕐",
                "🕑",
                "🕒",
                "🕓",
                "🕔",
                "🕕",
                "🕖",
                "🕗",
                "🕘",
                "🕙",
                "🕚",
            ],
            1000,
        )

        def signal_handler(signum: int, frame: str, spinner: Spinner) -> None:
            spinner.fail("Bye! Remember")
            spinner.stop()
            sys.exit(0)

        with yaspin(spinner=sp, sigmap={signal.SIGINT: signal_handler}, text=lifetimer):
            time.sleep(lifetimer.get_lifespan())

        print("Your last day of your life was gone.")
        print("See you again!")

        return 0


def execute_from_command_line(argv: Optional[str] = None) -> int:
    command = Command(argv)
    return command.execute()


if __name__ == "__main__":
    execute_from_command_line()
